#include <sys/stat.h>
#include <dirent.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <badpenguin-ansi.h>
#include <badpenguin-errors.h>
#include <badpenguin-fs.h>
#include "package-setup.h"
#include "parse-options.h"
#include "parse-request.h"
#include "request-version.h"
#include "setup-quit.h"


  char *lastFound, *lastInstalled, *lastDepot;

  // REQUEST action
  char *reqTemp = NULL;
  char *reqName = NULL;
  char *reqMin  = NULL;
  char *reqMax  = NULL;
  char *libName = NULL;
  char *libMin  = NULL;
  char *libMax  = NULL;


int RecurseDepotDir(char *subdir) {
  DIR *lpdir;
  int l;
  struct dirent *lpstdirent;
  char newdepot[240];

  lpdir = opendir(subdir);
  if (lpdir==NULL) {
    printWARNING("! impossibile aprire %s\n", subdir );
    normal_color();
    return 1;
  }
  lpstdirent = readdir(lpdir);
  l = strlen(reqName);
  lastFound = NULL;
  while (lpstdirent != NULL) {
    // check if it is a dir
    if ( 
	( lpstdirent->d_name[0] != '.') &&
	( 0 == is_directory(subdir,lpstdirent->d_name) )
    ) {
      sprintf( newdepot, "%s/%s", subdir, lpstdirent->d_name );
      RecurseDepotDir(newdepot);
    } else
    if (
        ( 0 == strncasecmp(reqName, lpstdirent->d_name, l)) &&
	( 0 == is_regularfile(subdir,lpstdirent->d_name) )
    ) {
      // Check if it is better of the installed copy ...
      if (gostOptions.verbose>1) printf("- trovato: %s\n", lpstdirent->d_name );	
      if (lastFound == NULL) {
        if ( lastInstalled == NULL ) {
	    lastFound = strdup(lpstdirent->d_name);
	    lastDepot = strdup(subdir);
	} else
        if (gostOptions.action == ACTION_REQUEST) {
          if ( request_version(lastInstalled,lpstdirent->d_name) < 0 ) {
            lastFound = strdup(lpstdirent->d_name);
            lastDepot = strdup(subdir);
          }
        } else {
          if ( request_version(lastInstalled,lpstdirent->d_name) < 0 ) {
            lastFound = strdup(lpstdirent->d_name);
            lastDepot = strdup(subdir);
          }
	}
      } else {
        // Check if it is better of the last copy ...
        if (gostOptions.action == ACTION_REQUEST) {
          if ( request_version(lastFound,lpstdirent->d_name) < 0 ) {
            free(lastFound);
            free(lastDepot);
            lastFound = strdup(lpstdirent->d_name);
            lastDepot = strdup(subdir);	  
          }
        } else {
          if ( request_version(lastFound,lpstdirent->d_name) < 0 ) {
            free(lastFound);
            free(lastDepot);
            lastFound = strdup(lpstdirent->d_name);
            lastDepot = strdup(subdir);	  
          }
        }
      }
    }
    lpstdirent = readdir(lpdir);
  }
  closedir(lpdir);
  return 0;
}

// Installa, Aggiorna, Ripristina il pacchetto
int setup_upgrade() {
  char *depot_name, *depot_file, *depot_time, *temp1;
  char buffer[255];
  FILE *fd;
  DIR *lpdirdepot;
  struct dirent *depotdirent;
  int ldepot;


  if (gostOptions.verbose>2) printf("- inizio upgrade pacchetto\n");

  // Source file name - not null
  if ( gostOptions.sourcefile == NULL ) {
    printERROR("! a filename for the package was not specified\n");
    normal_color();
    setup_quit(ERROR_PARAMETER);
  }
  // Target dir - not null
  if ( gostOptions.targetdir == NULL ) {
    printWARNING("! target dir is null then i assume interactive installation on /\n");
    normal_color();
    gostOptions.targetdir = strdup("/");
    gostOptions.interactive = 1;
  }
  // allocate spoolroot, spooldir, tempscript
  if (gostOptions.spoolroot == NULL) {
    gostOptions.spoolroot      = calloc(255,sizeof(char));
    strcpy(gostOptions.spoolroot, gostOptions.targetdir );
    strcat(gostOptions.spoolroot, DEFAULT_SPOOLROOT );
  }
  
  // Decode request action
  reqTemp = strdup(gostOptions.sourcefile);
  reqName = strtoken( &reqTemp, ":" );
  reqMin  = strtoken( &reqTemp, ":" );
  reqMax  = strtoken( &reqTemp, ":" );    
  libName = strtoken( &reqTemp, ":" );
  libMin  = strtoken( &reqTemp, ":" );
  libMax  = strtoken( &reqTemp, ":" );    
  // check if deps was resolved when action is REQUEST
  if (gostOptions.action == ACTION_REQUEST) {
    if (check_request(reqName,reqMin,reqMax,libName,libMin,libMax,0) >= 3) {
      printSUCCESS("- richiesta di %s gia' soddisfatta\n", reqName );
      normal_color();
      setup_quit(0);
    }    
  }
  
  // Cerca la versione + aggiornata attualmente installata
  lpdirdepot = opendir(gostOptions.spoolroot);
  if (lpdirdepot==NULL) {
    printERROR("! impossibile aprire %s\n", gostOptions.spoolroot );
    normal_color();
    setup_quit(ERROR_IO);
  }
  depotdirent = readdir(lpdirdepot);
  ldepot = strlen(reqName);
  lastInstalled = NULL;
  while (depotdirent != NULL) {
    if (
      ( 0 == strncasecmp(reqName, depotdirent->d_name, ldepot)) &&
      ( 0 == is_directory(gostOptions.spoolroot,depotdirent->d_name) )
    ) {
      if (lastInstalled==NULL) {
        lastInstalled = strdup(depotdirent->d_name);
      } else {
        if (gostOptions.action == ACTION_REQUEST) {
          if ( request_version(lastInstalled,depotdirent->d_name) < 0 ) {
            free(lastInstalled);
            lastInstalled = strdup(depotdirent->d_name);
          }
	} else {
          if ( request_version(lastInstalled,depotdirent->d_name) < 0 ) {
            free(lastInstalled);
            lastInstalled = strdup(depotdirent->d_name);
          }	
	}
      }
    }
    depotdirent = readdir(lpdirdepot);
  }
  closedir(lpdirdepot);
  // show results
  if (lastInstalled==NULL) {
    printf("- nessun pacchetto gia installato\n");
  } else {
    realloc( lastInstalled, (strlen(lastInstalled) + 5) * sizeof(char));
    strcat( lastInstalled, ".bpp" );
    printf("- pacchetto installato: %s\n", lastInstalled );
  }
  
  // Apre il DEPOT
  fd = fopen( PACKAGE_DEPOTS, "r");
  if (fd==NULL) {
    printERROR("! errore aprendo %s in lettura !\n", PACKAGE_DEPOTS);
    normal_color();
    return ERROR_IO;
  }
  // Legge i depot
  while (! feof(fd) ) {
    read_line( fd, buffer );
    temp1 = strdup(buffer);
    depot_name = strtok( temp1, " ");
    if (depot_name == NULL) continue;
    depot_file = strtok( NULL, " ");
    depot_time = strtok( NULL, " ");
    printf("> searching in %s \n", depot_name);
    // HTTP ?
    if ( 0 == strncasecmp( depot_name, "http:", 5) ) {
      printWARNING("! HTTP depots are not supported for now, skipping\n");
      normal_color();
      continue;
    }
    // FTP ?
    if ( 0 == strncasecmp( depot_name, "ftp:", 4) ) {
      printWARNING("! FTP depots are not supported for now, skipping\n");
      normal_color();
      continue;
    }
    // DIRECTORY
    if ( 0 != file_exists(depot_name) ) {
      printWARNING("- directory depot inesistente\n");
      normal_color();
      continue;
    }
    // Cerca ...
    if ( 0 != RecurseDepotDir(depot_name) ) continue;
    // continue to the next depot if nothing is found
    if (lastFound==NULL) continue;
    printf("- upgrade disponibile: %s in %s\n", lastFound, lastDepot );
    break;    
  }
  fclose(fd);

  // Not Found
  if (lastFound==NULL) {
    printERROR("! Nessun upgrade disponibile ");
    if (lastInstalled != NULL) {
      printf("a %s \n", lastInstalled);
    } else {
      printf("per %s*.bpp \n", reqName);
    }
    normal_color();
    setup_quit(ERROR_NOTFOUND);
  }

  // Installazione Upgrade  
  sprintf(buffer, "package-setup --no-header --install  %s/%s  %s", lastDepot, lastFound, gostOptions.targetdir );
  if (gostOptions.verbose>0) printf("- eseguo: %s\n", buffer);
  printf("/----- upgrading to %s -----\\\n", lastFound );
  ldepot = runshell(buffer);
  printf("\\----- upgraded  to %s -----/\n", lastFound );
  if (ldepot>0) ldepot=ERROR_IO;
  setup_quit( ldepot );
  return 0;
}
